/*
 *  Copyright CNRS-AIST
 *
 *  Authors:  Fumio Kanehiro, Florent Lamiraux, Olivier Stasse
 */
#include <stdio.h>
#include <iostream>

#include "jrl/dynamics/dynamicsfactory.hh"
#include "hrp2-dynamics/hrp2OptHumanoidDynamicRobot.h"

#include "TestingSuite.h"

using namespace std;

void hrp2Opt::TestingSuite::dv3d(vector3d &av3d, ostream &os) 
{
  for(unsigned int i=0;i<3;i++)
    os << av3d(i) << " ";
  os << endl;
}

void hrp2Opt::TestingSuite::dm3d(const matrix3d &todisplay, ostream &os, string shifttab)
{
  for(unsigned int i=0;i<3;i++)
    {
      for(unsigned int j=0;j<3;j++)
	{
	  os << MAL_S3x3_MATRIX_ACCESS_I_J(todisplay,i,j) << " " ;
	}
      os << endl;
      if (i!=2)
	os << shifttab;
    }
}

void hrp2Opt::TestingSuite::dm4d(const matrix4d &todisplay, ostream &os, string shifttab)
{
  for(unsigned int i=0;i<4;i++)
    {
      for(unsigned int j=0;j<4;j++)
	{
	  os << MAL_S4x4_MATRIX_ACCESS_I_J(todisplay,i,j) << " " ;
	}
      os << endl;
      if (i!=3)
	os << shifttab;
    }
}

void hrp2Opt::TestingSuite::DisplayBody(CjrlBody *aBody, string &shifttab)
{
  
  cout << shifttab << "Related body informations" << endl;
  vector3d alcm = aBody->localCenterOfMass();
  cout << shifttab << "Local center of Mass : ";
  dv3d(alcm,cout);
  matrix3d aim = aBody->inertiaMatrix();
  cout << shifttab << "Inertia Matrix: " ;
  dm3d(aim,cout,shifttab);
  cout << shifttab << "mass: " << aBody->mass() << endl;
      
}
void hrp2Opt::TestingSuite::RecursiveDisplayOfJoints(CjrlJoint *aJoint, 
			      unsigned int verbosedisplay,
			      unsigned int ldepth)
{
  if (aJoint==0)
    return;

  int NbChildren = aJoint->countChildJoints();
  string shifttab="";
  for(unsigned int i=0;i<ldepth;i++)
	shifttab+=" ";
  
  cout << shifttab << "Rank : " 
       << aJoint->rankInConfiguration() << endl;
  cout << shifttab << "Number of child  :" 
       << aJoint->countChildJoints() << endl;
  cout << shifttab << "Nb of degree of freedom " 
       <<  aJoint->numberDof() << endl;
  

  if (verbosedisplay>3)
    {
      cout << shifttab << "Initial Position " ;
      matrix4d iP = aJoint->initialPosition();
      dm4d(iP,cout,shifttab);
      
      cout << shifttab << "CurrentTransformation ";
      matrix4d cT = aJoint->currentTransformation();
      dm4d(cT,cout,shifttab);

      // Limits
      std::cout << shifttab << "llimit: " 
		<< aJoint->lowerBound(0)*180/M_PI << " " 
		<< shifttab << "ulimit: " 
		<< aJoint->upperBound(0)*180/M_PI << " " << endl;

      std::cout << shifttab << "lvlimit: " 
		<< aJoint->lowerVelocityBound(0)*180/M_PI << " " 
		<< shifttab << "uvlimit: " 
		<< aJoint->upperVelocityBound(0)*180/M_PI << " " << endl;
      
      // Path from the root to this joint.
      std::vector<CjrlJoint*> JointsFromRootToHere = aJoint->jointsFromRootToThis();
      cout << shifttab << "Nb of nodes: " << JointsFromRootToHere.size() << endl
	   << shifttab << "Joint from root to here:" << endl << shifttab;
      for(unsigned int i=0;i<JointsFromRootToHere.size();i++)
	cout << JointsFromRootToHere[i]->rankInConfiguration() << " ";
      cout << endl;

      // Current state of the joint.

      // Rigid velocity:
      CjrlRigidVelocity aRV = aJoint->jointVelocity();
      cout << shifttab << "Linear Velocity ";
      vector3d av3d = aRV.linearVelocity();
      dv3d(av3d,cout);
      cout << shifttab << "Angular Velocity ";
      av3d = aRV.rotationVelocity();
      dv3d(av3d,cout);

      // Rigit Acceleration.
      CjrlRigidAcceleration aRA = aJoint->jointAcceleration();
      cout << shifttab << "Linear Acceleration ";
      av3d = aRA.linearAcceleration();
      dv3d(av3d,cout);
      cout << shifttab << "Angular Acceleration ";
      av3d = aRA.rotationAcceleration();
      dv3d(av3d,cout);
      
      CjrlBody * aBody = aJoint->linkedBody();
      DisplayBody(aBody,shifttab);
    }

  if (NbChildren!=0)
    {
      cout << shifttab << "***********************************************" << endl;
      cout << shifttab << " Display Now information related to children :" << endl;
      
      for(int i=0;i<NbChildren;i++)
	{
	  // Returns a const so we have to force the casting/
	  RecursiveDisplayOfJoints(aJoint->childJoint(i),verbosedisplay,ldepth+1); 
	}
    }
}


void hrp2Opt::TestingSuite::DisplayDynamicRobotInformation(CjrlDynamicRobot *aDynamicRobot)
{
  std::vector<CjrlJoint *> aVec = aDynamicRobot->jointVector();
  int r = aVec.size();
  cout << "Number of joints :" << r << endl;
  
}

void hrp2Opt::TestingSuite::DisplayMatrix(MAL_MATRIX(,double) &aJ)
{
  for(int i=0;i<MAL_MATRIX_NB_ROWS(aJ);i++)
    {
      for(unsigned int j=0;j<MAL_MATRIX_NB_COLS(aJ);j++)
	{
	  if (aJ(i,j)==0.0)
	    printf("0 ");
	  else
	    printf("%10.5f ",aJ(i,j));
	}
      printf("\n");
    }

}

void hrp2Opt::TestingSuite::GoDownTree(const CjrlJoint * startJoint)
{
  std::cout << "joint ranked :" << startJoint->rankInConfiguration() << std::endl;
  std::cout << "llimit: " << startJoint->lowerBound(0)*180/M_PI << " " 
	    << "ulimit: " << startJoint->upperBound(0)*180/M_PI << " " << endl;
  std::cout << startJoint->currentTransformation() << std::endl;
  
  if (startJoint->countChildJoints()!=0)
    {
      const CjrlJoint * childJoint = startJoint->childJoint(0);
      GoDownTree(childJoint);
    }
}
