#include <stdlib.h>
#include <string>

#include <ostream>
#include <fstream>

#ifndef WIN32
#include <sys/time.h>
#endif
#include <time.h>

#include "jrl/dynamics/dynamicsfactory.hh"
#include "hrp2-dynamics/hrp2OptHumanoidDynamicRobot.h"

#include "TestingSuite.h"

using namespace std;
using namespace dynamicsJRLJapan;
using namespace std;
using namespace dynamicsJRLJapan;

#include "TaskJacobian.c"



int main(int argc, char *argv[])
{
  string aSpecificitiesFileName;
  string aPath;
  string aName;
  string aMapFromJointToRank;

  if (argc!=5)
    {
      cerr << "  This program takes 4 arguments: " << endl;
      cerr << "./testhrp2Dynamics  Path_to_vrml_file  HRP2_Vrml_File_name "<< endl;
      cerr << "  HRP2_Specificities_xml_File  HRP2_Map_Joint_2_rank_File" << endl;
      exit(-1);
	}
  else 
    {
      aSpecificitiesFileName = argv[3];
      aPath=argv[1];
      aName=argv[2];
      aMapFromJointToRank=argv[4];
    }
  dynamicsJRLJapan::ObjectFactory aRobotDynamicsObjectConstructor;
  
  Chrp2OptHumanoidDynamicRobot * aHDR = new Chrp2OptHumanoidDynamicRobot(&aRobotDynamicsObjectConstructor);
  

  if (aHDR==0)
    { 
      cerr<< "Dynamic cast on HDR failed " << endl;
      exit(-1);
    }
  string RobotFileName = aPath+aName;
  dynamicsJRLJapan::parseOpenHRPVRMLFile(*aHDR,RobotFileName,aMapFromJointToRank,aSpecificitiesFileName);
  
  // Display tree of the joints.
  CjrlJoint* rootJoint = aHDR->rootJoint();  

  int NbOfDofs = aHDR->numberDof();
  std::cout << "NbOfDofs :" << NbOfDofs << std::endl;
  if (NbOfDofs==0)
    {
      cerr << "Empty Robot..."<< endl;
      return -1;
    }
  MAL_VECTOR_DIM(aCurrentConf,double,NbOfDofs);
  int lindex=0;
  for(int i=0;i<6;i++)
    aCurrentConf[lindex++] = 0.0;
  
  for(int i=0;i<(NbOfDofs-6 < 40 ? NbOfDofs-6 : 40) ;i++)
    aCurrentConf[lindex++] = 0.0;
  //2*M_PI*((double)rand()/(double)RAND_MAX)-M_PI
  
  aHDR->currentConfiguration(aCurrentConf);

  MAL_VECTOR_DIM(aCurrentVel,double,NbOfDofs); 

  lindex=0;
  
#ifndef WIN32
  struct timeval currenttime;
  gettimeofday(&currenttime,0);
  srand(currenttime.tv_usec);
#else
  srand( (unsigned)time( NULL ) );
#endif

  for(int i=0;i<NbOfDofs;i++)
    aCurrentVel[lindex++] = 0;
  
  MAL_S3_VECTOR(ZMPval,double);

  aHDR->currentVelocity(aCurrentVel);
  //  aHDR->setComputeZMP(true);
  string inProperty="ComputeZMP"; string aValue="true";
  aHDR->setProperty(inProperty,aValue);
  aHDR->computeForwardKinematics();
  ZMPval = aHDR->zeroMomentumPoint();
  cout << "First value of ZMP : " << ZMPval <<endl;
  cout << "Should be equal to the CoM: " << aHDR->positionCenterOfMass() << endl;

  hrp2Opt::TestingSuite::TestInertiaMatrix(aHDR);

  /* Jacobian methods. */
  std::vector<CjrlJoint *> aVec = aHDR->jointVector();
  
  CjrlJoint  * aJoint = aVec[3]; // Try to get the hand.
  aJoint->computeJacobianJointWrtConfig();

  MAL_MATRIX(,double) aJ = aJoint->jacobianJointWrtConfig();
  
  //  DisplayMatrix(aJ);
  cout << "****************************" << endl;
  rootJoint->computeJacobianJointWrtConfig();
  aJ = rootJoint->jacobianJointWrtConfig();  
  cout << "Rank of Root: " << rootJoint->rankInConfiguration() << endl;

  //  DisplayMatrix(aJ);

  /* Center of mass Jacobian */ 
  hrp2Opt::TestingSuite::TestJacobianOfCoM(aHDR);

  /* Recursive display */
  hrp2Opt::TestingSuite::RecursiveDisplayOfJoints(rootJoint,10);

  cout << "****************************" << endl;

  // Test rank of the left hand.
  cout << "Rank of the left hand "<< endl;
  cout << aHDR->leftWrist()->rankInConfiguration() << endl;

  MAL_VECTOR_FILL(aCurrentVel,0.0);
  MAL_VECTOR_DIM(aCurrentAcc,double,NbOfDofs);
  MAL_VECTOR_FILL(aCurrentAcc,0.0);

  // This is mandatory for this implementation of computeForwardKinematics
  // to compute the derivative of the momentum.
  {
    string inProperty[4]={"TimeStep","ComputeAcceleration",
			  "ComputeBackwardDynamics", "ComputeZMP"};
    string inValue[4]={"0.005","false","false","true"};
    for(unsigned int i=0;i<4;i++)
      aHDR->setProperty(inProperty[i],inValue[i]);

  }
  for(int i=0;i<4;i++)
    {
      aHDR->currentVelocity(aCurrentVel);
      aHDR->currentAcceleration(aCurrentAcc);
      aHDR->computeForwardKinematics();
      ZMPval = aHDR->zeroMomentumPoint();
	  const vector3d com = aHDR->positionCenterOfMass();
	  if(ZMPval[0] == com[0] && ZMPval[1] == com[1])
	    {
	      cout << i << "-th {x,y} values of ZMP:(" << ZMPval[0] << "," << ZMPval[1] << ")";
	      cout << " CoM: (" << com[0] << ","<< com[1] << ") ";
	      cout << "; Z_zmp=" << ZMPval[2]	<< ", Z_com=" << com[2] <<endl;
	  }
	  else
	  {
		cout << i << "-th {x,y} values of ZMP : " << ZMPval <<endl;
		cout << "Should be equal to the {x,y} values of CoM: " << com << endl;
	  }      
    }

  // Test of the getSpecializedInverseKinematics 
  CjrlJoint *waist = aHDR->waist();
  CjrlJoint *lankle = aHDR->leftAnkle();

  matrix4d waistpose;
  matrix4d leftanklepose;
  MAL_S4x4_MATRIX_SET_IDENTITY(waistpose);
  MAL_S4x4_MATRIX_SET_IDENTITY(leftanklepose);

  // Trying to find back the half-sitting pose.
  MAL_S4x4_MATRIX_ACCESS_I_J(leftanklepose,1,3)=0.095;
  MAL_S4x4_MATRIX_ACCESS_I_J(leftanklepose,2,3)=-0.54;
  
  vectorN legsq;
  MAL_VECTOR_FILL(legsq,0.0);
  cout << "Specialized Inverse Kinematics: Finding back half-stting" << endl << "Before calling: q=(" << legsq << ")" <<endl;
  aHDR->getSpecializedInverseKinematics(*waist,*lankle,waistpose,leftanklepose,legsq);
  cout << "After calling : " << legsq*180/M_PI << endl;
  delete aHDR;
  
}
