/*
 * \file Functions to display informations on HRP-2.
 *  Copyright CNRS-AIST
 *
 *  Authors:  Olivier Stasse
 */

#ifndef _TESTING_SUITE_HRP2OPT_H_
# define _TESTING_SUITE_HRP2OPT_H_
# include "hrp2-dynamics/hrp2OptHumanoidDynamicRobot.h"
# include "abstract-robot-dynamics/humanoid-dynamic-robot.hh"

namespace hrp2Opt
{
  namespace TestingSuite
  {
    /*! \brief Provides inertia matrix output
      and comparison between optimized and regular
      computation */
    void TestInertiaMatrix(Chrp2OptHumanoidDynamicRobot * aHDR);

    /*! \brief Test the specialized inverse kinematics
      and more specificically the left leg. */
    void TestgetSpecializedInverseKinematics(Chrp2OptHumanoidDynamicRobot *aHDR);

    /*! \brief Create a HRP-2 robot */
    void createHRP2(const std::string &inPath);

    /*! \brief Provides Jacobian of CoM output
      and comparison between optimized and regular
      computation. */
    void TestJacobianOfCoM(Chrp2OptHumanoidDynamicRobot *aHDR);

    /*! \name Display functions
      @{
     */
    void dv3d(vector3d &av3d, std::ostream &os);

    void dm3d(const matrix3d &todisplay, std::ostream &os, std::string shifttab);

    void dm4d(const matrix4d &todisplay, std::ostream &os, std::string shifttab);

    void DisplayBody(CjrlBody *aBody, std::string &shifttab);

    void RecursiveDisplayOfJoints(CjrlJoint *aJoint,
			      unsigned int verbosedisplay=0,
				  unsigned int ldepth=0);

    void DisplayDynamicRobotInformation(CjrlDynamicRobot *aDynamicRobot);
    void DisplayMatrix(MAL_MATRIX(,double) &aJ);

    void GoDownTree(const CjrlJoint * startJoint);
    /*! @} */
  }
}

#endif /* _TESTING_SUITE_HRP2OPT_H_ */
