/*
 * \file Creating an HRP-2, set its position to zero 
 * and display the hands positions.
 *
 *  Copyright CNRS-AIST
 *
 *  Authors:  Olivier Stasse
 */

#include "TestingSuite.h"
#include "jrl/dynamics/dynamicsfactory.hh"
#include "hrp2-dynamics/hrp2OptHumanoidDynamicRobot.h"

#define M4_IJ MAL_S4x4_MATRIX_ACCESS_I_J
void hrp2Opt::TestingSuite::createHRP2 ( const std::string& inPath )
{
    dynamicsJRLJapan::ObjectFactory jrlRobotFactory;
    Chrp2OptHumanoidDynamicRobot * attJrlRobot=0;
    
    attJrlRobot = new Chrp2OptHumanoidDynamicRobot ( &jrlRobotFactory );
    std::string linkJointFname,specificitiesFile;
    linkJointFname = inPath + "HRP2LinkJointRank.xml";
    specificitiesFile = inPath + "HRP2Specificities.xml";
    std::string modelname = inPath + "HRP2JRLmain.wrl";
    std::cout << "linkJointFname:" <<  linkJointFname  << std::endl;
    std::cout << "specificitiesFile:" <<  specificitiesFile  << std::endl;
    std::cout << "modelname:" << modelname << std::endl;

    dynamicsJRLJapan::parseOpenHRPVRMLFile ( *attJrlRobot, modelname,linkJointFname,specificitiesFile );
    std::cout << "Initialization done" << std::endl;

    std::string property,value;
    property="ComputeZMP"; value="false";attJrlRobot->setProperty ( property,value );
    property="TimeStep"; value="0.005";attJrlRobot->setProperty ( property,value );
    property="ComputeAccelerationCoM"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeBackwardDynamics"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeMomentum"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeAcceleration"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeVelocity"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeSkewCoM"; value="false";attJrlRobot->setProperty ( property,value );
    property="ComputeCoM"; value="true";attJrlRobot->setProperty ( property,value );

    unsigned int nDof = attJrlRobot->numberDof();

    MAL_VECTOR_DIM(zeros,double,nDof);
    MAL_VECTOR_FILL(zeros,0.0);

    attJrlRobot->currentConfiguration ( zeros );
    attJrlRobot->currentVelocity ( zeros );
    attJrlRobot->currentAcceleration ( zeros );
    attJrlRobot->computeForwardKinematics();


    for ( unsigned int i=23;i<29;i++ )
    {
        const matrix4d& jointT = attJrlRobot->jointVector() [i]->currentTransformation();
        std::cout << "RHAND Mechanism joint " << i-23 << ":   "<< M4_IJ(jointT,0,3) <<" "<< M4_IJ(jointT,1,3) <<" "<<M4_IJ(jointT,2,3)<< std::endl;
    }
    
    for ( unsigned int i=35;i<41;i++ )
    {
        const matrix4d& jointT = attJrlRobot->jointVector() [i]->currentTransformation();
        std::cout << "LHAND Mechanism joint " << i-35 << ":   "<< M4_IJ(jointT,0,3) <<" "<< M4_IJ(jointT,1,3) <<" "<<M4_IJ(jointT,2,3)<< std::endl;
    }
}
